'''
Created on 2022-11-26

@author: wf
'''
from yprinciple.target import Target
from meta.metamodel import Topic, TopicLink
from datetime import datetime
from yprinciple.version import Version

class SMWTarget(Target):
    """
    a specialized generation target for Semantic MediaWiki 
    """
    @classmethod
    def getSMWTargets(cls):
        targets={
            "category": CategoryTarget("Category","archive"),
            "concept": ConceptTarget("Concept","puzzle"),
            "form": FormTarget("Form","form-select"),
            "help": HelpTarget("Help","help-box"),
            "listOf": ListOfTarget("List of","format-list-bulleted"),
            "template": TemplateTarget("Template","file-document"),
            "properties": PropertyMultiTarget("Properties","alpha-p-circle",is_multi=True),
            "python": Target("Python","snake")
        }
        for target_key,target in targets.items():
            target.target_key=target_key
        return targets
    
    def i18n(self,text:str)->str:
        """
        return the internationalized version of the given text
        
        Args:
            text(str): the text to internationalize
            
        Returns:
            str: the internationalized version of the text
        """
        # @TODO implement language support for non english wikis
        return text;
    
    def topicHeader(self,topic:'Topic')->str:
        """
        get the topic header for the given topic
        
        Args:
            topic(Topic): the topic to generate a header for
            
        Returns:
            str: the markup to be generated
        """
        markup=f"""{{{{#ask: [[Topic name::{topic.name}]]
|mainlabel=-
|?Topic icon = icon
|? = Topic
|?Topic name = name
|?Topic pluralName = pluralName
|?Topic documentation = documentation
}}}}
"""
        return markup
    
    def seealso(self,topic:Topic)->str:
        """
        generate wiki markup to to show relevant links for a topic
        """
        markup=f"""see also
* [[List of {topic.pluralName}]]
* [[Help:{topic.name}]]
* [[Concept:{topic.name}]]
* [[:Category:{topic.name}]]
* [[:Template:{topic.name}]]
* [[:Form:{topic.name}]] 
topic links:"""
        for topicLink in topic.sourceTopicLinks.values():
            markup+=f"* [[:Category:{topicLink.targetTopic.name}]]\n"
        return markup
        
    def askSort(self,topic:Topic)->str:
        """
        generate the sort clause for an SMW ask query
        
        Args:
            topic(Topic): the topic to generate wiki markup for
                
        Returns:
            str: the generated wiki markup 
        """
        sort=""
        order=""
        delim=""
        sortproperties=topic.sortProperties()
        for prop in sortproperties:
            direction="ascending" if getattr(prop,"sortAscending",True) else "descending"
            sort+=delim+f"{topic.name} {prop.name}"
            order+=delim+direction
            delim=","
            pass
        sortClause=f"|sort={sort}\n" if sort else ""
        orderClause=f"|order={order}\n" if order else ""
        markup=f"{sortClause}{orderClause}"
        return markup
    
    def copyright(self)->str:
        """
        get the copyright markup
        
        Returns:
            str: the copyright markup
        """
        markup="""<!--
  --     Copyright (C) 2015-2022 BITPlan GmbH
  -- 
  --     Pater-Delp-Str. -- 1
  --     D-47877 -- Willich-Schiefbahn
  -- 
  --     http://www.bitplan.com
  --
  -- 
-->"""
        return markup
    
    def profiWiki(self)->str:
        """
        markup for profiWiki
        """
        markup="[https://wiki.bitplan.com/index.php/ProfiWiki BITPlan Y-Prinzip ProfiWiki]"
        return markup
       
    def bitplanumlci(self,fontSize:int=12)->str:
        """
        create plantuml skin params for BITPlan corporate identity
        
        Args:
            fontSize(int): the font size to use
            
        Returns:
            str: the wiki markup to be generated
        """
        currentYear = datetime.now().year
        markup=f"""' BITPlan Corporate identity skin params
' Copyright (c) 2015-{currentYear} BITPlan GmbH
' see https://wiki.bitplan.com/index.php/PlantUmlSkinParams#BITPlanCI
' skinparams generated by {Version.name}
"""
        skinparams= ["note", "component", "package", "usecase","activity","classAttribute","interface","class","object"]
        for skinparam in skinparams:
            markup+=f"""skinparam {skinparam} {{
  BackGroundColor #FFFFFF
  FontSize {fontSize}
  ArrowColor #FF8000
  BorderColor #FF8000
  FontColor black
  FontName Technical
}}
"""
        markup+="""hide Circle
' end of skinparams '"""
        return markup
    
    def plantUmlRelation(self,topicLink:TopicLink)->str:
        """
        generate wiki markup for a TopicLink/relation
        
        Args:    
            topicLink(TopicLink): the topicLink to generate the relation for
            
        Returns:
            str: the wiki markup to generate
        """
        sourceMany="*" if topicLink.sourceMultiple else "1"
        targetMany="*" if topicLink.targetMultiple else "1"
        markup=f"""{topicLink.source} "{topicLink.sourceRole} ({sourceMany})" -- "{topicLink.targetRole}({targetMany})" {topicLink.target}\n"""
        return markup
    
    def uml(self,title:str,topic:'Topic',output_format:str='svg')->str:
        """
        get the uml (plantuml) markup for  the given topic
        
        Args:
            topic(Topic): the topic to generate a header for
            output_format(str): the output format to use - default: svg
            
        Returns:
            str: the plantuml markup to be generated
            
        """
        markup=f"""=== {title} ===
<uml format='{output_format}'>
title {topic.name}
note as {topic.name}DiagramNote
Copyright (c) 2015-2022 BITPlan GmbH
[[http://www.bitplan.com]]
end note
note as {topic.name}Note
{topic.documentation}
end note
class {topic.name} {{
"""
        for prop in topic.properties.values():
            markup+=f"  {prop.type} {prop.name}\n"
        markup+=f"""}}
{topic.name}Note .. {topic.name}
"""
        # Relations/Topic Links        
        for topicLink in topic.sourceTopicLinks.values(): 
            markup+=f"{self.plantUmlRelation(topicLink)}"
        for topicLink in topic.targetTopicLinks.values(): 
            markup+=f"{self.plantUmlRelation(topicLink)}"
        markup+=f"""{self.bitplanumlci(12)}
</uml>"""
        return markup
    
class CategoryTarget(SMWTarget):
    """
    the target to generate "Category" pages 
    
    see https://wiki.bitplan.com/index.php/SiDIFTemplates#category
    """
    
    def generate(self,topic:'Topic')->str:
        """
        generate a category page for the given topic
        
        e.g. https://wiki.bitplan.com/index.php/Category:Topic
        
        see https://wiki.bitplan.com/index.php/SiDIFTemplates#category
        
        Args:
            topic(Topic): the topic to generate wiki markup for
            
        Returns:
            str: the generated wiki markup 
        """
        markup=f"""__NOTOC__
{{{{#ask: [[Topic name::{topic.name}]] | ?Topic wikiDocumentation= | mainlabel=-}}}}
{topic.getPluralName()} may be added and edited with the form [[Form:{topic.name}]]
* [[List of {topic.getPluralName()}]]
<div class="toccolours mw-collapsible mw-collapsed" style="width:1024px">
{topic.name} {self.i18n("description")}
<div class="mw-collapsible-content">
{self.uml("uml",topic)}
* [[Help:{topic.name}]]
* [[Concept:{topic.name}]]
* [[:Template:{topic.name}]]
* [[:Form:{topic.name}]]
=== Properties ===
"""
        for prop in topic.properties.values():
            markup+=f"* [[Property:{topic.name} {prop.name}]]\n"
        markup+="""</div>
</div>"""
        return markup
        
class ConceptTarget(SMWTarget):
    """
    the target to generate "Concept" pages 
    
    see https://wiki.bitplan.com/index.php/SiDIFTemplates#concept
    """
    
    def generate(self,topic:'Topic')->str:
        """
        generate a result for the given topic
        
        see https://wiki.bitplan.com/index.php/SiDIFTemplates#concept
        
        Args:
            topic(Topic): the topic to generate wiki markup for
            
        Returns:
            str: the generated wiki markup 
        """
        conceptClause=f"""[[{topic.name} {topic.conceptProperty.name}::+]]""" \
            if hasattr(topic,"conceptProperty") \
            else f"""[[Category:{topic.name}]]"""
        markup=f"""{{{{Topic
|name={topic.name}
|pluralName={topic.getPluralName()}
|icon={topic.icon}
|iconUrl={topic.iconUrl}
|documentation={topic.documentation}
|wikiDocumentation={topic.wikiDocumentation}
|defaultstoremode={topic.defaultstoremode}
|listLimit={topic.getListLimit()}
|cargo={getattr(topic,"cargo","false")}
|context={topic.context}
|storemode=property
}}}}
{{{{Topic
|viewmode=masterdetail
|storemode=none
}}}}
{{{{#forminput:form=Property|button text=add Property}}}}
=== Documentation ===
{topic.wikiDocumentation}
{self.uml("uml",topic)}

{{{{#concept:
{conceptClause}
 |{topic.pluralName}
{self.seealso(topic)}
}}}}
[[Category:{topic.name}]]
"""
        return markup


    
class HelpTarget(SMWTarget):
    """
    the help Target
    """

    def generate(self,topic:'Topic')->str:
        """
        generate a result for the given topic
        
        see https://wiki.bitplan.com/index.php/SiDIFTemplates#help
        
        Args:
            topic(Topic): the topic to generate wiki markup for
            
        Returns:
            str: the generated wiki markup 
        """
        markup=f"""[[File:Help_Icon.png|right]]
== Help for {topic.name} ==
{self.topicHeader(topic)}
=== Documentation ===
{topic.wikiDocumentation}
=== Example {topic.pluralName} ===
{{{{#ask: [[Concept:{topic.name}]]
}}}}
=== Properties ===
{{{{#ask: [[Concept:Property]][[Property topic::Concept:{topic.name}]]
| ?Property documentation = documentation
| ?Property type = type
| ?Property name = name
| ?Property label = label
| ?Property allowedValues = allowedValues
| ?Property mandatory = mandatory
| ?Property uploadable = uploadable
|format=table
}}}}
{self.uml("uml",topic)}
{self.seealso(topic)}
[[Category:{topic.name}]]
"""
        return markup
    
class FormTarget(SMWTarget):
    """
    the target to generate "Form" pages 
    e.g. https://wiki.bitplan.com/index.php/Form:Topic
    
    see https://wiki.bitplan.com/index.php/SiDIFTemplates#form
    
    """
    
    def generate(self,topic:Topic)->str:
        """
        generate the form page for the given topic

        Args:
            topic(Topic): the topic to generate wiki markup for
                
        Returns:
            str: the generated wiki markup 
        """
        markup=f"""<noinclude>
This is the {self.profiWiki()}-Form for "{topic.name}".
=== see also ===
{self.seealso(topic)}
</noinclude><includeonly></includeonly>
        """
        return markup
    
class ListOfTarget(SMWTarget):
    """
    the target to generate "List of" pages 
    e.g. https://wiki.bitplan.com/index.php/List_of_Topics
    
    see https://wiki.bitplan.com/index.php/SiDIFTemplates#listof
    
    """
    
    def generate(self,topic:Topic)->str:
        """
        generate the list of page for the given topic

        Args:
            topic(Topic): the topic to generate wiki markup for
                
        Returns:
            str: the generated wiki markup 
        """
        markup=f"""__NOCACHE__
{self.topicHeader(topic)}
== {topic.getPluralName()} ==
{{{{#ask: [[Concept:{topic.name}]]|format=count}}}}
{{{{#forminput:form={topic.name}|button text=add {topic.name}}}}}
{{{{#ask: [[Concept:{topic.name}]]
|mainlabel={topic.name}"""
        for prop in topic.properties.values():
            markup+=f"| ?{topic.name}  {prop.name} = {prop.name}\n"
        markup+=f"""{self.askSort(topic)}}}}}
[[:Category:{topic.name}]]
    """
        return markup
    
class TemplateTarget(SMWTarget):
    """
    the template Target
    """

    def generate(self,topic:'Topic')->str:
        """
        generate a template for the given topic
        
        see https://wiki.bitplan.com/index.php/SiDIFTemplates#template
        
        Args:
            topic(Topic): the topic to generate wiki markup for
            
        Returns:
            str: the generated wiki markup 
        """
        markup=f"""<noinclude>{self.copyright()}
=== see also ===
{self.seealso(topic)}
[[Category:Template]]
</noinclude><includeonly></includeonly>"""
        return markup
    
class PropertyMultiTarget(SMWTarget):
    """
    the Property Multi Target
    """
    
    